@@ -1,25 +1,29 @@
 # -*- coding: utf-8 -*-
 
 from __future__ import (absolute_import, division, print_function,
-    unicode_literals)
-from pyramid.compat             import text_type as unicode
-from pyramid.i18n               import TranslationStringFactory
-from pyramid.security           import Allow
-from datetime                   import datetime
-from datetime                   import timedelta
-from datetime                   import date
-from sqlalchemy.ext.declarative import declared_attr
-from sqlalchemy.ext.hybrid      import hybrid_property
-from sqlalchemy                 import or_
-from sqlalchemy                 import func
+                        unicode_literals)
+try:
+    from urllib.parse import urlencode  # Python 3
+except ImportError:
+    from urllib import urlencode  # Python 2
 
-from hem.text                   import generate_random_string
-from hem.text                   import pluralize
-from hem.db                     import get_session
+from pyramid.compat import text_type as unicode
+from pyramid.i18n import TranslationStringFactory
+from pyramid.security import Allow
+from datetime import datetime
+from datetime import timedelta
+from datetime import date
+from sqlalchemy.ext.declarative import declared_attr
+from sqlalchemy.ext.hybrid import hybrid_property
+from sqlalchemy import or_
+from sqlalchemy import func
+
+from hem.text import generate_random_string
+from hem.text import pluralize
+from hem.db import get_session
 
 import cryptacular.bcrypt
 import re
-import urllib
 import hashlib
 import sqlalchemy as sa
 
@@ -96,7 +100,12 @@ class BaseModel(object):
     def get_by_id(cls, request, id):
         """Gets an object by its primary key."""
         session = get_session(request)
-        return session.query(cls).filter(cls.id == id).first()
+        pk = getattr(cls, cls._idAttribute)
+        return session.query(cls).filter(pk == id).first()
+
+
+def three_days_from_now():
+    return datetime.utcnow() + timedelta(days=3)
 
 
 class ActivationMixin(BaseModel):
@@ -111,39 +120,32 @@ class ActivationMixin(BaseModel):
     @declared_attr
     def code(self):
         """A random hash that is valid only once."""
-        return sa.Column(sa.Unicode(30), nullable=False, unique=True)
+        return sa.Column(sa.Unicode(30), nullable=False,
+                         unique=True,
+                         default=generate_random_string)
 
     @declared_attr
     def valid_until(self):
         """How long will the activation key last"""
-        return sa.Column(sa.DateTime, nullable=False)
+        return sa.Column(sa.DateTime, nullable=False,
+                         default=three_days_from_now)
 
     @declared_attr
     def created_by(self):
         """The system that generated the activation key"""
-        return sa.Column(sa.Unicode(30), nullable=False)
+        return sa.Column(sa.Unicode(30), nullable=False,
+                         default='web')
 
     @classmethod
     def get_by_code(cls, request, code):
         session = get_session(request)
         return session.query(cls).filter(cls.code == code).first()
 
-    def __init__(self, created_by='web', valid_until=None):
-        """Create a new activation. *valid_until* is a datetime.
-        It defaults to 3 days from current day.
-        """
-        self.code = generate_random_string(12)
-        self.created_by = created_by
-
-        if valid_until:
-            self.valid_until = valid_until
-        else:
-            self.valid_until = datetime.utcnow() + timedelta(days=3)
-
 
 def default_security_code():
     return generate_random_string(12)
 
+
 class UserMixin(BaseModel):
     @declared_attr
     def username(self):
@@ -228,7 +230,7 @@ class UserMixin(BaseModel):
 
     @property
     def is_activated(self):
-        if self.activation_id == None:
+        if self.activation_id is None:
             return True
 
         return False
@@ -245,14 +247,13 @@ class UserMixin(BaseModel):
 
         return unicode(crypt.encode(password + self.salt))
 
-    def gravatar_url(self, default='mm'):
-        """ returns user gravatar url """
-        # construct the url
-        h = hashlib.md5(self.email.encode('utf8').lower()).hexdigest()
-        base_url = "https://secure.gravatar.com/avatar/%s?%s"
-        gravatar_url = base_url % (h, urllib.urlencode({'d': default}))
-
-        return gravatar_url
+    def gravatar_url(self, default='mm', size=80, cacheable=True):
+        '''Returns a Gravatar image URL for this user.'''
+        base = "http://www.gravatar.com/avatar/" if cacheable else \
+            "https://secure.gravatar.com/avatar/"
+        return base + \
+            hashlib.md5(self.email.encode('utf8').lower()).hexdigest() + \
+            "?" + urlencode({'d': default, 's': str(size)})
 
     @classmethod
     def generate_random_password(cls, chars=12):
@@ -291,7 +292,7 @@ class UserMixin(BaseModel):
         user = cls.get_by_email(request, email)
 
         if user:
-            valid = cls.validate_user(request, user, password)
+            valid = cls.validate_user(user, password)
 
             if valid:
                 return user
@@ -330,7 +331,7 @@ class UserMixin(BaseModel):
         if not user:
             return None
 
-        if user.password == None:
+        if user.password is None:
             valid = False
         else:
             valid = crypt.check(user.password, password + user.salt)
@@ -387,11 +388,11 @@ class GroupMixin(BaseModel):
 class UserGroupMixin(BaseModel):
     @declared_attr
     def group_id(self):
-        return sa.Column(sa.Integer,
+        return sa.Column(
+            sa.Integer,
             sa.ForeignKey('%s.%s' % (
-                    GroupMixin.__tablename__,
-                    self._idAttribute
-                )
+                GroupMixin.__tablename__,
+                self._idAttribute)
             )
         )
 
@@ -399,14 +400,18 @@ class UserGroupMixin(BaseModel):
     def user_id(self):
         return sa.Column(
             sa.Integer,
-            sa.ForeignKey('%s.%s' % (
-                    UserMixin.__tablename__,
-                    self._idAttribute
-                ),
-                onupdate='CASCADE',
-                ondelete='CASCADE'
-            ),
+            sa.ForeignKey('%s.%s' % (UserMixin.__tablename__,
+                                     self._idAttribute),
+                          onupdate='CASCADE',
+                          ondelete='CASCADE'),
         )
 
     def __repr__(self):
         return '<UserGroup: %s, %s>' % (self.group_name, self.user_id,)
+
+__all__ = [
+    k for k, v in locals().items()
+    if (
+        isinstance(v, type) and issubclass(v, BaseModel)
+    )
+]
