@@ -1,11 +1,11 @@
 # -*- coding: utf-8 -*-
 
 from __future__ import (absolute_import, division, print_function,
-    unicode_literals)
+                        unicode_literals)
 import colander
 import deform
 
-from pyramid_deform import CSRFSchema
+from hem.schemas import CSRFSchema
 
 
 class ProfileSchema(CSRFSchema):
