@@ -3,10 +3,7 @@
 from __future__ import (absolute_import, division, print_function,
     unicode_literals)
 from pyramid import testing
-
-from mock import Mock
-from mock import patch
-
+from mock import Mock, patch
 from horus.tests import UnitTestBase
 
 
@@ -14,14 +11,19 @@ class TestAuthController(UnitTestBase):
     def test_auth_controller_extensions(self):
         from horus.views        import AuthController
         from horus.interfaces   import IUserClass
-        from horus.tests.models import User
         from horus.interfaces   import ILoginSchema
         from horus.interfaces   import ILoginForm
         from horus.interfaces   import IActivationClass
+        from horus.interfaces   import IUIStrings
+        from horus.strings      import UIStringsBase
+        from horus.tests.models import User
         from horus.tests.models import Activation
+
         self.config.registry.registerUtility(Activation, IActivationClass)
 
         self.config.add_route('index', '/')
+        self.config.registry.settings['horus.login_redirect'] = 'index'
+        self.config.registry.settings['horus.logout_redirect'] = 'index'
 
         request = testing.DummyRequest()
 
@@ -31,6 +33,7 @@ class TestAuthController(UnitTestBase):
         schema = Mock()
         form = Mock()
 
+        self.config.registry.registerUtility(UIStringsBase, IUIStrings)
         self.config.registry.registerUtility(User, IUserClass)
         self.config.registry.registerUtility(schema, ILoginSchema)
         self.config.registry.registerUtility(form, ILoginForm)
@@ -41,15 +44,17 @@ class TestAuthController(UnitTestBase):
         assert form.called
 
     def test_login_loads(self):
-        from horus.views          import AuthController
-        from horus.interfaces     import IUserClass
-        from horus.tests.models   import User
+        from horus.views        import AuthController
+        from horus.interfaces   import IUserClass
+        from horus.tests.models import User
         from horus.interfaces   import IActivationClass
         from horus.tests.models import Activation
         self.config.registry.registerUtility(Activation, IActivationClass)
 
         self.config.registry.registerUtility(User, IUserClass)
         self.config.add_route('index', '/')
+        self.config.registry.settings['horus.login_redirect'] = 'index'
+        self.config.registry.settings['horus.logout_redirect'] = 'index'
         self.config.include('horus')
 
         request = testing.DummyRequest()
@@ -61,8 +66,8 @@ class TestAuthController(UnitTestBase):
 
     def test_login_redirects_if_logged_in(self):
         from horus.views import AuthController
-        from horus.interfaces     import IUserClass
-        from horus.tests.models   import User
+        from horus.interfaces   import IUserClass
+        from horus.tests.models import User
         from horus.interfaces   import IActivationClass
         from horus.tests.models import Activation
         self.config.registry.registerUtility(Activation, IActivationClass)
@@ -70,6 +75,8 @@ class TestAuthController(UnitTestBase):
         self.config.registry.registerUtility(User, IUserClass)
         self.config.add_route('index', '/')
         self.config.include('horus')
+        self.config.registry.settings['horus.login_redirect'] = 'index'
+        self.config.registry.settings['horus.logout_redirect'] = 'index'
 
         request = testing.DummyRequest()
         request.user = Mock()
@@ -79,10 +86,10 @@ class TestAuthController(UnitTestBase):
         assert response.status_int == 302
 
     def test_login_fails_empty(self):
-        """ Make sure we can't login with empty credentials"""
+        """Make sure we can't log in with empty credentials."""
         from horus.views import AuthController
-        from horus.interfaces     import IUserClass
-        from horus.tests.models   import User
+        from horus.interfaces   import IUserClass
+        from horus.tests.models import User
         from horus.interfaces   import IActivationClass
         from horus.tests.models import Activation
         self.config.registry.registerUtility(Activation, IActivationClass)
@@ -90,6 +97,8 @@ class TestAuthController(UnitTestBase):
         self.config.registry.registerUtility(User, IUserClass)
         self.config.add_route('index', '/')
         self.config.include('horus')
+        self.config.registry.settings['horus.login_redirect'] = 'index'
+        self.config.registry.settings['horus.logout_redirect'] = 'index'
 
         request = testing.DummyRequest(post={
             'submit': True,
@@ -109,8 +118,8 @@ class TestAuthController(UnitTestBase):
     def test_csrf_invalid_fails(self):
         """ Make sure we can't login with a bad csrf """
         from horus.views import AuthController
-        from horus.interfaces     import IUserClass
-        from horus.tests.models   import User
+        from horus.interfaces   import IUserClass
+        from horus.tests.models import User
         from horus.interfaces   import IActivationClass
         from horus.tests.models import Activation
         self.config.registry.registerUtility(Activation, IActivationClass)
@@ -118,6 +127,8 @@ class TestAuthController(UnitTestBase):
         self.config.registry.registerUtility(User, IUserClass)
         self.config.add_route('index', '/')
         self.config.include('horus')
+        self.config.registry.settings['horus.login_redirect'] = 'index'
+        self.config.registry.settings['horus.logout_redirect'] = 'index'
 
         request = self.get_csrf_request(post={
                     'submit': True,
@@ -138,8 +149,8 @@ class TestAuthController(UnitTestBase):
     def test_login_fails_bad_credentials(self):
         """ Make sure we can't login with bad credentials"""
         from horus.views import AuthController
-        from horus.interfaces     import IUserClass
-        from horus.tests.models   import User
+        from horus.interfaces   import IUserClass
+        from horus.tests.models import User
         from horus.interfaces   import IActivationClass
         from horus.tests.models import Activation
         self.config.registry.registerUtility(Activation, IActivationClass)
@@ -147,6 +158,8 @@ class TestAuthController(UnitTestBase):
         self.config.registry.registerUtility(User, IUserClass)
         self.config.add_route('index', '/')
         self.config.include('horus')
+        self.config.registry.settings['horus.login_redirect'] = 'index'
+        self.config.registry.settings['horus.logout_redirect'] = 'index'
 
         request = self.get_csrf_request(post={
                 'submit': True,
@@ -154,23 +167,23 @@ class TestAuthController(UnitTestBase):
                 'password': 'test123',
             }, request_method='POST')
 
-        flash = Mock()
-        request.session.flash = flash
-
         view = AuthController(request)
-        view.login()
-
-        flash.assert_called_with('Invalid username or password.', 'error')
+        with patch('horus.views.FlashMessage') as FlashMessage:
+            view.login()
+            FlashMessage.assert_called_with(request,
+                "Invalid username or password.", kind="error")
 
     def test_login_succeeds(self):
         """Make sure we can log in."""
         from horus.tests.models import User
-        from horus.interfaces     import IUserClass
+        from horus.interfaces   import IUserClass
         from horus.interfaces   import IActivationClass
         from horus.tests.models import Activation
         self.config.registry.registerUtility(Activation, IActivationClass)
 
         self.config.registry.registerUtility(User, IUserClass)
+        self.config.registry.settings['horus.login_redirect'] = 'index'
+        self.config.registry.settings['horus.logout_redirect'] = 'index'
 
         admin = User(username='sontek', email='sontek@gmail.com')
         admin.password = 'foo'
@@ -197,7 +210,7 @@ class TestAuthController(UnitTestBase):
     def test_inactive_login_fails(self):
         """Make sure we can't log in with an inactive user."""
         from horus.tests.models import User
-        from horus.interfaces     import IUserClass
+        from horus.interfaces   import IUserClass
         from horus.interfaces   import IActivationClass
         from horus.tests.models import Activation
         self.config.registry.registerUtility(Activation, IActivationClass)
@@ -212,6 +225,8 @@ class TestAuthController(UnitTestBase):
         from horus.views import AuthController
         self.config.add_route('index', '/')
         self.config.include('horus')
+        self.config.registry.settings['horus.login_redirect'] = 'index'
+        self.config.registry.settings['horus.logout_redirect'] = 'index'
 
         request = self.get_csrf_request(post={
                 'submit': True,
@@ -219,43 +234,41 @@ class TestAuthController(UnitTestBase):
                 'password': 'foo',
             }, request_method='POST')
 
-        flash = Mock()
-
-        request.session.flash = flash
-
         view = AuthController(request)
-        view.login()
-
-        flash.assert_called_with(
-            'Your account is not active, please check your e-mail.',
-            'error')
+        with patch('horus.views.FlashMessage') as FlashMessage:
+            view.login()
+            FlashMessage.assert_called_with(request,
+                'Your account is not active, please check your e-mail.',
+                kind='error')
 
     def test_logout(self):
+        from horus.strings      import UIStringsBase as Str
         from horus.views        import AuthController
         from horus.tests.models import User
-        from horus.interfaces import IUserClass
+        from horus.interfaces   import IUserClass
         from horus.interfaces   import IActivationClass
         from horus.tests.models import Activation
         self.config.registry.registerUtility(Activation, IActivationClass)
         self.config.registry.registerUtility(User, IUserClass)
         self.config.add_route('index', '/')
         self.config.include('horus')
+        self.config.registry.settings['horus.login_redirect'] = 'index'
+        self.config.registry.settings['horus.logout_redirect'] = 'index'
         request = testing.DummyRequest()
 
-        flash = Mock()
         invalidate = Mock()
-
         request.user = Mock()
         request.session = Mock()
         request.session.invalidate = invalidate
-        request.session.flash = flash
 
         view = AuthController(request)
         with patch('horus.views.forget') as forget:
             with patch('horus.views.HTTPFound') as HTTPFound:
-                view.logout()
-                flash.assert_called_with('Logged out successfully.',
-                    'success')
+                with patch('horus.views.FlashMessage') as FlashMessage:
+                    view.logout()
+                    FlashMessage.assert_called_with(request,
+                        Str.logout, kind="success")
+
                 forget.assert_called_with(request)
                 assert invalidate.called
                 assert HTTPFound.called
@@ -270,11 +283,13 @@ class TestRegisterController(UnitTestBase):
         from horus.interfaces           import IRegisterForm
         from horus.interfaces           import IUserClass
         from horus.tests.models         import User
+        from horus.interfaces   import IUIStrings
+        from horus.strings      import UIStringsBase
         from horus.interfaces   import IActivationClass
         from horus.tests.models import Activation
         self.config.registry.registerUtility(Activation, IActivationClass)
-
         self.config.registry.registerUtility(User, IUserClass)
+        self.config.registry.registerUtility(UIStringsBase, IUIStrings)
 
         self.config.add_route('index', '/')
 
@@ -298,16 +313,18 @@ class TestRegisterController(UnitTestBase):
         assert form.called
 
     def test_register_controller_extensions_without_mail(self):
-        from horus.views import RegisterController
-        from horus.interfaces import IRegisterSchema
-        from horus.interfaces import IRegisterForm
-        from horus.interfaces           import IUserClass
-        from horus.tests.models         import User
+        from horus.views        import RegisterController
+        from horus.interfaces   import IRegisterSchema
+        from horus.interfaces   import IRegisterForm
+        from horus.interfaces   import IUserClass
+        from horus.interfaces   import IUIStrings
+        from horus.strings      import UIStringsBase
+        from horus.tests.models import User
         from horus.interfaces   import IActivationClass
         from horus.tests.models import Activation
         self.config.registry.registerUtility(Activation, IActivationClass)
-
         self.config.registry.registerUtility(User, IUserClass)
+        self.config.registry.registerUtility(UIStringsBase, IUIStrings)
 
         self.config.add_route('index', '/')
 
@@ -370,6 +387,8 @@ class TestRegisterController(UnitTestBase):
         self.config.registry.registerUtility(DummyMailer(), IMailer)
 
         self.config.add_route('index', '/')
+        self.config.registry.settings['horus.login_redirect'] = 'index'
+        self.config.registry.settings['horus.logout_redirect'] = 'index'
 
         request = testing.DummyRequest()
         request.user = Mock()
@@ -448,7 +467,6 @@ class TestRegisterController(UnitTestBase):
         from horus.tests.models         import User
         from horus.interfaces           import IActivationClass
         from horus.tests.models         import Activation
-        from horus.exceptions           import RegistrationFailure
         self.config.registry.registerUtility(Activation, IActivationClass)
 
         self.config.registry.registerUtility(User, IUserClass)
@@ -472,11 +490,10 @@ class TestRegisterController(UnitTestBase):
             'email': 'sontek@gmail.com'
         }, request_method='POST')
 
-        flash = Mock()
-        request.session.flash = flash
-
-        controller = RegisterController(request)
-        self.assertRaises(RegistrationFailure, controller.register)
+        view = RegisterController(request)
+        adict = view.register()
+        assert isinstance(adict, dict)
+        assert adict['errors']
 
     def test_register_no_email_validation(self):
         from horus.views import RegisterController
@@ -484,8 +501,8 @@ class TestRegisterController(UnitTestBase):
         from pyramid_mailer.interfaces import IMailer
         from hem.interfaces import IDBSession
         from horus.events import NewRegistrationEvent
-        from horus.interfaces           import IUserClass
-        from horus.tests.models         import User
+        from horus.interfaces import IUserClass
+        from horus.tests.models import User
         from horus.interfaces   import IActivationClass
         from horus.tests.models import Activation
         self.config.registry.registerUtility(Activation, IActivationClass)
@@ -514,21 +531,16 @@ class TestRegisterController(UnitTestBase):
             'email': 'sontek@gmail.com'
         }, request_method='POST')
 
-        flash = Mock()
-        request.session.flash = flash
-
         request.user = Mock()
 
-        controller = RegisterController(request)
-        response = controller.register()
-
+        view = RegisterController(request)
+        with patch('horus.views.FlashMessage') as FlashMessage:
+            response = view.register()
+            FlashMessage.assert_called_with(request,
+                view.Str.registration_done, kind="success")
         assert response.status_int == 302
-
         user = User.get_by_username(request, 'admin')
-
         assert user.is_activated == True
-        flash.assert_called_with(
-            'You have been registered, you may log in now!', 'success')
 
     def test_registration_craps_out(self):
         from horus.views                import RegisterController
@@ -561,9 +573,6 @@ class TestRegisterController(UnitTestBase):
             'email': 'sontek@gmail.com'
         }, request_method='POST')
 
-        flash = Mock()
-        request.session.flash = flash
-
         request.user = Mock()
         controller = RegisterController(request)
 
@@ -573,8 +582,8 @@ class TestRegisterController(UnitTestBase):
         from horus.views import RegisterController
         from pyramid_mailer.interfaces import IMailer
         from pyramid_mailer.mailer import DummyMailer
-        from horus.interfaces           import IUserClass
-        from horus.tests.models         import User
+        from horus.interfaces import IUserClass
+        from horus.tests.models import User
         from horus.interfaces   import IActivationClass
         from horus.tests.models import Activation
         self.config.registry.registerUtility(Activation, IActivationClass)
@@ -782,19 +791,19 @@ class TestForgotPasswordController(UnitTestBase):
         assert response.status_int == 302
 
     def test_forgot_password_valid_user(self):
-        from horus.views import ForgotPasswordController
-        from pyramid_mailer.interfaces import IMailer
-        from pyramid_mailer.mailer import DummyMailer
+        from horus.views                import ForgotPasswordController
+        from pyramid_mailer.interfaces  import IMailer
+        from pyramid_mailer.mailer      import DummyMailer
         from horus.interfaces           import IUserClass
         from horus.tests.models         import User
-
         self.config.registry.registerUtility(User, IUserClass)
 
         self.config.add_route('index', '/')
         self.config.include('horus')
         self.config.registry.registerUtility(DummyMailer(), IMailer)
 
-        user = User(username='sontek', password='temp', email='sontek@gmail.com')
+        user = User(username='sontek', password='temp',
+            email='sontek@gmail.com')
         user.password = 'foo'
 
         self.session.add(user)
@@ -806,14 +815,12 @@ class TestForgotPasswordController(UnitTestBase):
 
         request.user = None
 
-        flash = Mock()
-        request.session.flash = flash
-
         view = ForgotPasswordController(request)
-        response = view.forgot_password()
 
-        flash.assert_called_with(
-            'Please check your e-mail to reset your password.', 'success')
+        with patch('horus.views.FlashMessage') as FlashMessage:
+            response = view.forgot_password()
+            FlashMessage.assert_called_with(request,
+                view.Str.reset_password_email_sent, kind="success")
         assert response.status_int == 302
 
     def test_forgot_password_invalid_password(self):
@@ -829,7 +836,8 @@ class TestForgotPasswordController(UnitTestBase):
         self.config.include('horus')
         self.config.registry.registerUtility(DummyMailer(), IMailer)
 
-        user = User(username='sontek', password='temp', email='sontek@gmail.com')
+        user = User(username='sontek', password='temp',
+            email='sontek@gmail.com')
         user.password = 'foo'
 
         self.session.add(user)
@@ -862,7 +870,8 @@ class TestForgotPasswordController(UnitTestBase):
         self.config.include('horus')
         self.config.registry.registerUtility(DummyMailer(), IMailer)
 
-        user = User(username='sontek', password='temp', email='sontek@gmail.com')
+        user = User(username='sontek', password='temp',
+            email='sontek@gmail.com')
         user.password = 'foo'
         user.activation = Activation()
 
@@ -924,9 +933,6 @@ class TestForgotPasswordController(UnitTestBase):
 
         request.user = None
 
-        flash = Mock()
-        request.session.flash = flash
-
         def handle_password_reset(event):
             request = event.request
             session = request.registry.getUtility(IDBSession)
@@ -956,8 +962,8 @@ class TestForgotPasswordController(UnitTestBase):
         self.config.include('horus')
         self.config.registry.registerUtility(DummyMailer(), IMailer)
 
-
-        user = User(username='sontek', password='temp', email='sontek@gmail.com')
+        user = User(username='sontek', password='temp',
+            email='sontek@gmail.com')
         user.password = 'foo'
         user.activation = Activation()
 
@@ -978,9 +984,6 @@ class TestForgotPasswordController(UnitTestBase):
 
         request.user = None
 
-        flash = Mock()
-        request.session.flash = flash
-
         view = ForgotPasswordController(request)
         response = view.reset_password()
 
@@ -1002,7 +1005,8 @@ class TestForgotPasswordController(UnitTestBase):
         self.config.include('horus')
         self.config.registry.registerUtility(DummyMailer(), IMailer)
 
-        user = User(username='sontek', password='temp', email='sontek@gmail.com')
+        user = User(username='sontek', password='temp',
+            email='sontek@gmail.com')
         user.password = 'foo'
         user.activation = Activation()
 
@@ -1018,9 +1022,6 @@ class TestForgotPasswordController(UnitTestBase):
 
         request.user = None
 
-        flash = Mock()
-        request.session.flash = flash
-
         view = ForgotPasswordController(request)
 
         response = view.reset_password()
@@ -1043,7 +1044,8 @@ class TestForgotPasswordController(UnitTestBase):
         self.config.include('horus')
         self.config.registry.registerUtility(DummyMailer(), IMailer)
 
-        user = User(username='sontek', password='temp', email='sontek@gmail.com')
+        user = User(username='sontek', password='temp',
+            email='sontek@gmail.com')
         user.password = 'foo'
         user.activation = Activation()
 
@@ -1079,8 +1081,7 @@ class TestProfileController(UnitTestBase):
         self.config.add_route('index', '/')
         self.config.include('horus')
 
-        user = User(username='sontek',
-                email='sontek@gmail.com')
+        user = User(username='sontek', email='sontek@gmail.com')
         user.password = 'temp'
         self.session.add(user)
         self.session.flush()
@@ -1088,9 +1089,6 @@ class TestProfileController(UnitTestBase):
         request = testing.DummyRequest()
         request.user = Mock()
 
-        flash = Mock()
-        request.session.flash = flash
-
         request.matchdict = Mock()
         get = Mock()
         get.return_value = user.id
@@ -1124,9 +1122,6 @@ class TestProfileController(UnitTestBase):
         request = testing.DummyRequest()
         request.user = Mock()
 
-        flash = Mock()
-        request.session.flash = flash
-
         request.matchdict = Mock()
         get = Mock()
         get.return_value = 99
@@ -1168,9 +1163,6 @@ class TestProfileController(UnitTestBase):
         get.return_value = user.id
         request.matchdict.get = get
 
-        flash = Mock()
-        request.session.flash = flash
-
         view = ProfileController(request)
 
         response = view.edit_profile()
@@ -1211,9 +1203,6 @@ class TestProfileController(UnitTestBase):
 
         request.user = user
 
-        flash = Mock()
-        request.session.flash = flash
-
         request.matchdict = Mock()
         get = Mock()
         get.return_value = user.id
@@ -1272,9 +1261,6 @@ class TestProfileController(UnitTestBase):
         get.return_value = user.id
         request.matchdict.get = get
 
-        flash = Mock()
-        request.session.flash = flash
-
         view = ProfileController(request)
 
         view.edit_profile()
