@@ -1,45 +1,55 @@
 # -*- coding: utf-8 -*-
 
 from __future__ import (absolute_import, division, print_function,
-    unicode_literals)
-from pyramid.view           import view_config
-from pyramid.url            import route_url
-from pyramid.security       import remember
-from pyramid.security       import forget
+                        unicode_literals)
+from pyramid.view import view_config
+from pyramid.url import route_url
+from pyramid.security import remember
+from pyramid.security import forget
 from pyramid.httpexceptions import HTTPFound
 from pyramid.httpexceptions import HTTPNotFound
-from pyramid.settings       import asbool
+from pyramid.settings import asbool
 
-from pyramid_mailer         import get_mailer
+from pyramid_mailer import get_mailer
 from pyramid_mailer.message import Message
 
-from horus.interfaces       import IUserClass
-from horus.interfaces       import IActivationClass
-from horus.interfaces       import ILoginForm
-from horus.interfaces       import ILoginSchema
-from horus.interfaces       import IRegisterForm
-from horus.interfaces       import IRegisterSchema
-from horus.interfaces       import IForgotPasswordForm
-from horus.interfaces       import IForgotPasswordSchema
-from horus.interfaces       import IResetPasswordForm
-from horus.interfaces       import IResetPasswordSchema
-from horus.interfaces       import IProfileForm
-from horus.interfaces       import IProfileSchema
-from horus.events           import NewRegistrationEvent
-from horus.events           import RegistrationActivatedEvent
-from horus.events           import PasswordResetEvent
-from horus.events           import ProfileUpdatedEvent
-from horus.models           import _
-from horus.exceptions       import AuthenticationFailure
-from horus.exceptions       import RegistrationFailure
-from horus.httpexceptions   import HTTPBadRequest
-from hem.db                 import get_session
+from hem.db import get_session
+from horus.interfaces import IUserClass
+from horus.interfaces import IActivationClass
+from horus.interfaces import IUIStrings
+from horus.interfaces import ILoginForm
+from horus.interfaces import ILoginSchema
+from horus.interfaces import IRegisterForm
+from horus.interfaces import IRegisterSchema
+from horus.interfaces import IForgotPasswordForm
+from horus.interfaces import IForgotPasswordSchema
+from horus.interfaces import IResetPasswordForm
+from horus.interfaces import IResetPasswordSchema
+from horus.interfaces import IProfileForm
+from horus.interfaces import IProfileSchema
+from horus.events import NewRegistrationEvent
+from horus.events import RegistrationActivatedEvent
+from horus.events import PasswordResetEvent
+from horus.events import ProfileUpdatedEvent
+from horus.lib import FlashMessage
+from horus.models import _
+from horus.exceptions import AuthenticationFailure
+from horus.httpexceptions import HTTPBadRequest
 
 import colander
 import deform
 import pystache
 
 
+def get_config_route(request, config_key):
+    settings = request.registry.settings
+
+    try:
+        return request.route_url(settings[config_key])
+    except KeyError:
+        return settings[config_key]
+
+
 def authenticated(request, userid):
     """Sets the auth cookies and redirects to the page defined
     in horus.login_redirect, which defaults to a view named 'index'.
@@ -49,10 +59,10 @@ def authenticated(request, userid):
     autologin = asbool(settings.get('horus.autologin', False))
 
     if not autologin:
-        request.session.flash(_('Logged in successfully.'), 'success')
+        Str = request.registry.getUtility(IUIStrings)
+        FlashMessage(request, Str.authenticated, kind='success')
 
-    login_redirect_route = settings.get('horus.login_redirect', 'index')
-    location = route_url(login_redirect_route, request)
+    location = get_config_route(request, 'horus.login_redirect')
 
     return HTTPFound(location=location, headers=headers)
 
@@ -67,18 +77,19 @@ def create_activation(request, user):
 
     db.flush()
 
+    # TODO Create a hook for the app to give us body and subject!
+    # TODO We don't need pystache just for this!
     body = pystache.render(
-        _("Please activate your e-mail address by visiting {{ link }}"),
+        _("Please validate your email and activate your account by visiting:\n"
+            "{{ link }}"),
         {
             'link': request.route_url('activate', user_id=user.id,
                                       code=user.activation.code)
         }
     )
-
-    subject = _("Please activate your e-mail address!")
+    subject = _("Please activate your account!")
 
     message = Message(subject=subject, recipients=[user.email], body=body)
-
     mailer = get_mailer(request)
     mailer.send(message)
 
@@ -92,8 +103,10 @@ class BaseController(object):
     def __init__(self, request):
         self._request = request
         self.settings = request.registry.settings
-        self.User = request.registry.getUtility(IUserClass)
-        self.Activation = request.registry.getUtility(IActivationClass)
+        getUtility = request.registry.getUtility
+        self.User = getUtility(IUserClass)
+        self.Activation = getUtility(IActivationClass)
+        self.Str = getUtility(IUIStrings)
         self.db = get_session(request)
 
 
@@ -106,22 +119,23 @@ class AuthController(BaseController):
 
         form = request.registry.getUtility(ILoginForm)
 
-        self.login_redirect_view = route_url(
-            self.settings.get('horus.login_redirect', 'index'),
-            request
+        self.login_redirect_view = get_config_route(
+            request,
+            'horus.login_redirect'
         )
-        self.logout_redirect_view = route_url(
-            self.settings.get('horus.logout_redirect', 'index'),
-            request
+
+        self.logout_redirect_view = get_config_route(
+            request,
+            'horus.logout_redirect'
         )
+
         self.require_activation = asbool(
             self.settings.get('horus.require_activation', True)
         )
         self.allow_inactive_login = asbool(
             self.settings.get('horus.allow_inactive_login', False)
         )
-
-        self.form = form(self.schema)
+        self.form = form(self.schema, buttons=(self.Str.login_button,))
 
     def check_credentials(self, username, password):
         allow_email_auth = self.settings.get('horus.allow_email_auth', False)
@@ -129,7 +143,8 @@ class AuthController(BaseController):
         user = self.User.get_user(self.request, username, password)
 
         if allow_email_auth and not user:
-            user = self.User.get_by_email_password(username, password)
+            user = self.User.get_by_email_password(
+                self.request, username, password)
 
         if not user:
             raise AuthenticationFailure(_('Invalid username or password.'))
@@ -138,15 +153,10 @@ class AuthController(BaseController):
                 and not user.is_activated:
             raise AuthenticationFailure(
                 _('Your account is not active, please check your e-mail.'))
-
         return user
 
-    @view_config(
-            route_name='login'
-            , xhr=True
-            , accept="application/json"
-            , renderer='json'
-        )
+    @view_config(route_name='login', xhr=True, accept="application/json",
+                 renderer='json')
     def login_ajax(self):
         try:
             cstruct = self.request.json_body
@@ -174,18 +184,16 @@ class AuthController(BaseController):
         # on the user
         user_json = user.__json__(self.request)
 
-        return {
-            'status': 'okay'
-            , 'user': user_json
-        }
+        return {'status': 'okay',
+                'user': user_json}
 
     @view_config(route_name='login', renderer='horus:templates/login.mako')
     def login(self):
         if self.request.method == 'GET':
             if self.request.user:
                 return HTTPFound(location=self.login_redirect_view)
-
             return {'form': self.form.render()}
+
         elif self.request.method == 'POST':
             try:
                 controls = self.request.POST.items()
@@ -202,13 +210,15 @@ class AuthController(BaseController):
             try:
                 user = self.check_credentials(username, password)
             except AuthenticationFailure as e:
-                self.request.session.flash(str(e), 'error')
+                FlashMessage(self.request, str(e), kind='error')
                 return {
                     'form': self.form.render(appstruct=captured),
                     'errors': [e]
                 }
 
-            return authenticated(self.request, user.id)
+            self.request.user = user  # Please keep this line, my app needs it
+
+            return authenticated(self.request, user.id_value)
 
     @view_config(permission='view', route_name='logout')
     def logout(self):
@@ -216,9 +226,8 @@ class AuthController(BaseController):
         horus.logout_redirect, which defaults to a view named 'index'.
         """
         self.request.session.invalidate()
-        self.request.session.flash(_('Logged out successfully.'), 'success')
+        FlashMessage(self.request, self.Str.logout, kind='success')
         headers = forget(self.request)
-
         return HTTPFound(location=self.logout_redirect_view, headers=headers)
 
 
@@ -228,64 +237,55 @@ class ForgotPasswordController(BaseController):
 
         self.forgot_password_redirect_view = route_url(
             self.settings.get('horus.forgot_password_redirect', 'index'),
-            request
-        )
+            request)
         self.reset_password_redirect_view = route_url(
             self.settings.get('horus.reset_password_redirect', 'index'),
-            request
-        )
+            request)
 
     @view_config(route_name='forgot_password',
                  renderer='horus:templates/forgot_password.mako')
     def forgot_password(self):
-        schema = self.request.registry.getUtility(IForgotPasswordSchema)
-        schema = schema().bind(request=self.request)
+        req = self.request
+        schema = req.registry.getUtility(IForgotPasswordSchema)
+        schema = schema().bind(request=req)
 
-        form = self.request.registry.getUtility(IForgotPasswordForm)
+        form = req.registry.getUtility(IForgotPasswordForm)
         form = form(schema)
 
-        if self.request.method == 'GET':
-            if self.request.user:
+        if req.method == 'GET':
+            if req.user:
                 return HTTPFound(location=self.forgot_password_redirect_view)
+            else:
+                return {'form': form.render()}
 
-            return {'form': form.render()}
+        # From here on, we know it's a POST. Let's validate the form
+        controls = req.POST.items()
+        try:
+            captured = form.validate(controls)
+        except deform.ValidationFailure as e:
+            # This catches if the email does not exist, too.
+            return {'form': e.render(), 'errors': e.error.children}
 
-        elif self.request.method == 'POST':
-            try:
-                controls = self.request.POST.items()
-                captured = form.validate(controls)
-            except deform.ValidationFailure as e:
-                return {'form': e.render(), 'errors': e.error.children}
+        user = self.User.get_by_email(req, captured['email'])
+        activation = self.Activation()
+        self.db.add(activation)
+        user.activation = activation
+        Str = self.Str
 
-            email = captured['email']
+        # TODO: Generate msg in a separate method so subclasses can override
+        mailer = get_mailer(req)
+        username = getattr(user, 'short_name', '') or \
+            getattr(user, 'full_name', '') or \
+            getattr(user, 'username', '') or user.email
+        body = Str.reset_password_email_body.format(
+            link=route_url('reset_password', req, code=user.activation.code),
+            username=username, domain=req.application_url)
+        subject = Str.reset_password_email_subject
+        message = Message(subject=subject, recipients=[user.email], body=body)
+        mailer.send(message)
 
-            user = self.User.get_by_email(self.request, email)
-            activation = self.Activation()
-            self.db.add(activation)
-
-            user.activation = activation
-
-            if user:
-                mailer = get_mailer(self.request)
-                body = pystache.render(
-                    _("Someone has tried to reset your password, "
-                      "if this was you click here: {{ link }}"),
-                    {
-                        'link': route_url('reset_password', self.request,
-                                          code=user.activation.code)
-                    }
-                )
-
-                subject = _("Do you want to reset your password?")
-
-                message = Message(subject=subject, recipients=[user.email],
-                                  body=body)
-                mailer.send(message)
-
-        # we don't want to say "E-mail not registered" or anything like that
-        # because it gives spammers context
-        self.request.session.flash(
-            _('Please check your e-mail to reset your password.'), 'success')
+        FlashMessage(self.request, Str.reset_password_email_sent,
+            kind='success')
         return HTTPFound(location=self.reset_password_redirect_view)
 
     @view_config(route_name='reset_password',
@@ -327,16 +327,12 @@ class ForgotPasswordController(BaseController):
                     self.db.add(user)
                     self.db.delete(activation)
 
-                    self.request.registry.notify(
-                        PasswordResetEvent(self.request, user, password)
-                    )
-
-                    self.request.session.flash(
-                        _('Your password has been reset!'), 'success')
-
+                    FlashMessage(self.request, self.Str.reset_password_done,
+                                 kind='success')
+                    self.request.registry.notify(PasswordResetEvent(
+                        self.request, user, password))
                     location = self.reset_password_redirect_view
                     return HTTPFound(location=location)
-
         return HTTPNotFound()
 
 
@@ -349,9 +345,9 @@ class RegisterController(BaseController):
         form = request.registry.getUtility(IRegisterForm)
         self.form = form(self.schema)
 
-        self.register_redirect_view = route_url(
+        self.after_register_url = route_url(
             self.settings.get('horus.register_redirect', 'index'), request)
-        self.activate_redirect_view = route_url(
+        self.after_activate_url = route_url(
             self.settings.get('horus.activate_redirect', 'index'), request)
 
         self.require_activation = asbool(
@@ -360,72 +356,53 @@ class RegisterController(BaseController):
         if self.require_activation:
             self.mailer = get_mailer(request)
 
-    def create_user(self, email, username, password):
-        user = self.User.get_by_username_or_email(
-            self.request,
-            username,
-            email
-        )
-
-        if user:
-            if user.username == username:
-                raise RegistrationFailure(
-                    _('That username is already used.'))
-            elif user.email == email:
-                raise RegistrationFailure(
-                    _('That e-mail is already used.'))
-
-        user = self.User(username=username, email=email, password=password)
-        self.db.add(user)
-        return user
-
     @view_config(route_name='register',
                  renderer='horus:templates/register.mako')
     def register(self):
         if self.request.method == 'GET':
             if self.request.user:
-                return HTTPFound(location=self.register_redirect_view)
+                return HTTPFound(location=self.after_register_url)
             return {'form': self.form.render()}
-        elif self.request.method == 'POST':
-            try:
-                controls = self.request.POST.items()
-                captured = self.form.validate(controls)
-            except deform.ValidationFailure as e:
-                return {'form': e.render(), 'errors': e.error.children}
+        elif self.request.method != 'POST':
+            return
 
-            email = captured['email']
-            username = captured['username'].lower()
-            password = captured['password']
+        # If the request is a POST:
+        controls = self.request.POST.items()
+        try:
+            captured = self.form.validate(controls)
+        except deform.ValidationFailure as e:
+            return {'form': e.render(), 'errors': e.error.children}
 
-            try:
-                user = self.create_user(email, username, password)
-            except AuthenticationFailure as e:
-                self.request.session.flash(str(e), 'error')
+        # With the form validated, we know email and username are unique.
+        del captured['csrf_token']
+        user = self.persist_user(captured)
 
-            autologin = asbool(self.settings.get('horus.autologin', False))
+        autologin = asbool(self.settings.get('horus.autologin', False))
 
-            activation = None
-            if self.require_activation:
-                # SEND EMAIL ACTIVATION
-                create_activation(self.request, user)
-                self.request.session.flash(
-                    _('Please check your e-mail for an activation link.'),
-                    'success')
-            else:
-                if not autologin:
-                    self.request.session.flash(
-                        _('You have been registered, you may log in now!'),
-                        'success')
+        if self.require_activation:
+            # SEND EMAIL ACTIVATION
+            create_activation(self.request, user)
+            FlashMessage(self.request, self.Str.activation_check_email,
+                         kind='success')
+        elif not autologin:
+            FlashMessage(self.request, self.Str.registration_done,
+                         kind='success')
 
-            self.request.registry.notify(
-                NewRegistrationEvent(self.request, user, activation, captured)
-            )
+        self.request.registry.notify(NewRegistrationEvent(
+            self.request, user, None, controls))
+        if autologin:
+            self.db.flush()  # in order to get the id
+            return authenticated(self.request, user.id)
+        else:  # not autologin: user must log in just after registering.
+            return HTTPFound(location=self.after_register_url)
 
-            if autologin:
-                self.db.flush()  # in order to get the id
-                return authenticated(self.request, user.id)
-            else:  # not autologin: User must log in just after registering.
-                return HTTPFound(location=self.register_redirect_view)
+    def persist_user(self, controls):
+        '''To change how the user is stored, override this method.'''
+        # This generic method must work with any custom User class and any
+        # custom registration form:
+        user = self.User(**controls)
+        self.db.add(user)
+        return user
 
     @view_config(route_name='activate')
     def activate(self):
@@ -442,17 +419,13 @@ class RegisterController(BaseController):
 
             if user:
                 self.db.delete(activation)
-                self.db.add(user)
+                # self.db.add(user)  # not necessary
                 self.db.flush()
-
+                FlashMessage(self.request, self.Str.activation_email_verified,
+                             kind='success')
                 self.request.registry.notify(
-                    RegistrationActivatedEvent(self.request, user, activation)
-                )
-
-                self.request.session.flash(
-                    _('Your e-mail address has been verified.'), 'success')
-                return HTTPFound(location=self.activate_redirect_view)
-
+                    RegistrationActivatedEvent(self.request, user, activation))
+                return HTTPFound(location=self.after_activate_url)
         return HTTPNotFound()
 
 
@@ -513,8 +486,8 @@ class ProfileController(BaseController):
 
                 if email_user:
                     if email_user.id != user.id:
-                        self.request.session.flash(
-                            _('That e-mail is already used.'), 'error')
+                        FlashMessage(self.request,
+                            _('That e-mail is already used.'), kind='error')
                         return HTTPFound(location=self.request.url)
 
                 user.email = email
@@ -524,13 +497,12 @@ class ProfileController(BaseController):
             if password:
                 user.password = password
 
-            self.request.session.flash(_('Profile successfully updated.'),
-                                       'success')
+            FlashMessage(self.request, self.Str.edit_profile_done,
+                         kind='success')
 
             self.db.add(user)
 
             self.request.registry.notify(
                 ProfileUpdatedEvent(self.request, user, captured)
             )
-
             return HTTPFound(location=self.request.url)
